'use strict';

exports.__esModule = true;

var _dom = require('gleaf/lib/utils/dom');

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function getPosition(event) {
  return event.touches && event.touches.length > 0 ? event.touches[0] : event;
}

var Dragger = function () {
  function Dragger(dragger) {
    _classCallCheck(this, Dragger);

    this.dragger = dragger;
    this.cachedShiftX = 0;
    this.cachedShiftY = 0;
    this.startX = 0;
    this.startY = 0;
    this.mousemoveProxy = null;
    this.mouseupProxy = null;
    this.mousedownProxy = null;
  }

  Dragger.prototype.mousedown = function mousedown(event) {
    var target = event.target;

    if (target && target.nodeName === 'INPUT') {
      return;
    }

    var _getPosition = getPosition(event),
        clientX = _getPosition.clientX,
        clientY = _getPosition.clientY;

    this.mousemoveProxy = this.mousemove.bind(this);
    this.mouseupProxy = this.mouseup.bind(this);

    (0, _dom.addEvents)([{ dom: document, event: 'mousemove', callback: this.mousemoveProxy }, { dom: document, event: 'mouseup', callback: this.mouseupProxy }, { dom: document, event: 'touchmove', callback: this.mousemoveProxy }, { dom: document, event: 'touchend', callback: this.mouseupProxy }]);

    this.startX = this.dragger.offsetLeft;
    this.startY = this.dragger.offsetTop;

    this.cachedShiftX = clientX;
    this.cachedShiftY = clientY;
  };

  Dragger.prototype.mouseup = function mouseup(event) {
    (0, _dom.removeEvents)([{ dom: document, event: 'mousemove', callback: this.mousemoveProxy }, { dom: document, event: 'mouseup', callback: this.mouseupProxy }, { dom: document, event: 'touchmove', callback: this.mousemoveProxy }, { dom: document, event: 'touchend', callback: this.mouseupProxy }]);
    event.preventDefault();
  };

  Dragger.prototype.mousemove = function mousemove(event) {
    var _getPosition2 = getPosition(event),
        clientX = _getPosition2.clientX,
        clientY = _getPosition2.clientY;

    var mouseMoveRangeX = clientX - this.cachedShiftX;
    var mouseMoveRangeY = clientY - this.cachedShiftY;

    var left = this.startX + mouseMoveRangeX;
    var top = this.startY + mouseMoveRangeY;

    var areaWidth = this.dragger.parentElement.offsetWidth;
    var areaHeight = this.dragger.parentElement.offsetHeight;

    var dragerWidth = this.dragger.offsetWidth;
    var dragerHeight = this.dragger.offsetHeight;

    left = left <= 0 ? 0 : left >= areaWidth - dragerWidth ? areaWidth - dragerWidth : left;

    top = top <= 0 ? 0 : top >= areaHeight - dragerHeight ? areaHeight - dragerHeight : top;

    (0, _dom.setStyle)(this.dragger, {
      left: left + 'px',
      top: top + 'px',
      margin: 0
    });
    event.preventDefault();
  };

  Dragger.prototype.addDraggableListeners = function addDraggableListeners() {
    this.mousedownProxy = this.mousedown.bind(this);

    (0, _dom.addEvents)([{ dom: this.dragger, event: 'mousedown', callback: this.mousedownProxy }, { dom: this.dragger, event: 'touchstart', callback: this.mousedownProxy }]);
    return this;
  };

  Dragger.prototype.destroyed = function destroyed() {
    (0, _dom.removeEvents)([{ dom: this.dragger, event: 'mousedown', callback: this.mousedownProxy }, { dom: this.dragger, event: 'touchstart', callback: this.mousedownProxy }]);
  };

  return Dragger;
}();

exports.default = Dragger;