import { stopDefaultEvent, on, createElement, appendToBody, addClass, removeClass, setStyle, addClassList } from 'gleaf/src/utils/dom';

const getMask = function() {
  let maskDom = createElement('div');
  stopDefaultEvent(maskDom, 'touchmove');
  return maskDom;
};

class LayerManage {
  constructor(zIndex) {
    this.zIndex = zIndex;
    this.maskDom = null;
    this.modalList = [];
  }

  doOnMaskClick() {
    this.closeMask();
  }

  setMaskIn(modal, mask) {
    addClass(this.maskDom, 'v-mask-enter');
    setStyle(this.maskDom, { display: mask ? 'block' : 'none' });
    setStyle(this.maskDom, { zIndex: this.zIndex });
    setStyle(modal, { zIndex: this.zIndex + 1 });
    setTimeout(() => {
      removeClass(this.maskDom, 'v-mask-enter');
    }, 200);
  }
  openMask(modal, mask) {
    if (this.$isServer) return;
    this.modalList.push({ modal, mask });
    if (this.maskDom !== null) {
      this.setMaskIn(modal, mask);
      return;
    }
    let maskDom = getMask();
    addClassList(maskDom, ['v-mask', 'v-mask-enter']);
    setStyle(maskDom, { zIndex: this.zIndex, display: mask ? 'block' : 'none' });
    setStyle(modal, { zIndex: this.zIndex + 1 });
    on(maskDom, 'click', () => {
      this.doOnMaskClick && this.doOnMaskClick();
    });
    appendToBody(this.maskDom = maskDom);
    setTimeout(() => {
      removeClass(maskDom, 'v-mask-enter');
    }, 200);
  }

  closeMask() {
    this.modalList.pop();
    if (this.modalList.length === 0) {
      addClass(this.maskDom, 'v-mask-leave');
      setStyle(this.maskDom, { display: 'none' });
      setTimeout(() => {
        removeClass(this.maskDom, 'v-mask-leave');
      }, 200);
    } else {
      setStyle(this.maskDom, {
        zIndex: this.previousLayer(),
        display: this.modalList[this.modalList.length - 1].mask ? 'block' : 'none'
      });
    }
  }

  nextLayer() {
    return (this.zIndex += 2);
  }
  previousLayer() {
    return (this.zIndex -= 2);
  }

  getCurrentLayer() {
    return this.zIndex;
  }
}

export default new LayerManage(1998);
