
import { setStyle, addEvents, removeEvents } from 'gleaf/src/utils/dom';

function getPosition(event) {
  return event.touches && event.touches.length > 0
    ? event.touches[0]
    : event;
}

export default class Dragger {
  constructor(dragger) {
    this.dragger = dragger;
    this.cachedShiftX = 0;
    this.cachedShiftY = 0;
    this.startX = 0;
    this.startY = 0;
    this.mousemoveProxy = null;
    this.mouseupProxy = null;
    this.mousedownProxy = null;
  }

  mousedown(event) {
    let target = event.target;

    if (target && target.nodeName === 'INPUT') {
      return;
    }

    let { clientX, clientY } = getPosition(event);

    this.mousemoveProxy = this.mousemove.bind(this);
    this.mouseupProxy = this.mouseup.bind(this);

    addEvents([
      { dom: document, event: 'mousemove', callback: this.mousemoveProxy },
      { dom: document, event: 'mouseup', callback: this.mouseupProxy },
      { dom: document, event: 'touchmove', callback: this.mousemoveProxy },
      { dom: document, event: 'touchend', callback: this.mouseupProxy }
    ]);

    this.startX = this.dragger.offsetLeft;
    this.startY = this.dragger.offsetTop;

    this.cachedShiftX = clientX;
    this.cachedShiftY = clientY;
  }

  mouseup(event) {
    removeEvents([
      { dom: document, event: 'mousemove', callback: this.mousemoveProxy },
      { dom: document, event: 'mouseup', callback: this.mouseupProxy },
      { dom: document, event: 'touchmove', callback: this.mousemoveProxy },
      { dom: document, event: 'touchend', callback: this.mouseupProxy }
    ]);
    event.preventDefault();
  }

  mousemove(event) {
    let { clientX, clientY } = getPosition(event);

    let mouseMoveRangeX = clientX - this.cachedShiftX;
    let mouseMoveRangeY = clientY - this.cachedShiftY;

    let left = this.startX + mouseMoveRangeX;
    let top = this.startY + mouseMoveRangeY;

    let areaWidth = this.dragger.parentElement.offsetWidth;
    let areaHeight = this.dragger.parentElement.offsetHeight;

    let dragerWidth = this.dragger.offsetWidth;
    let dragerHeight = this.dragger.offsetHeight;

    left = left <= 0
      ? 0
      : left >= areaWidth - dragerWidth
        ? areaWidth - dragerWidth
        : left;

    top = top <= 0
      ? 0
      : top >= areaHeight - dragerHeight
        ? areaHeight - dragerHeight
        : top;

    setStyle(this.dragger, {
      left: left + 'px',
      top: top + 'px',
      margin: 0
    });
    event.preventDefault();
  }

  addDraggableListeners() {
    this.mousedownProxy = this.mousedown.bind(this);

    addEvents([
      { dom: this.dragger, event: 'mousedown', callback: this.mousedownProxy },
      { dom: this.dragger, event: 'touchstart', callback: this.mousedownProxy }
    ]);
    return this;
  }

  destroyed() {
    removeEvents([
      { dom: this.dragger, event: 'mousedown', callback: this.mousedownProxy },
      { dom: this.dragger, event: 'touchstart', callback: this.mousedownProxy }
    ]);
  }
}
